package sample.jpa;


import javax.ejb.EJBException;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;


@Stateless
@Remote(AccountInterface.class)
public class AccountBean implements AccountInterface { 
    
   
    @PersistenceContext(type=PersistenceContextType.TRANSACTION)    
    private EntityManager manager;
    
  
    @TransactionAttribute(TransactionAttributeType.REQUIRED)    
    public Account open(int accountNumber) {
        Account account = manager.find(Account.class, accountNumber);
        if(account == null){
        	account = new Account();
        	account.ownerName = "anonymous";
            account.accountNumber = accountNumber;
            manager.persist(account);
            return account;
        }else{
        	throw new EJBException("Account already exists..!!. Account Number = "+accountNumber);
        }
        
    }    
    
    @TransactionAttribute(TransactionAttributeType.REQUIRED)    
	public double getBalance(int accountNumber) {
    	 Account account = manager.find(Account.class, accountNumber);
        if(account==null)
            throw new EJBException("Account not found..!!. Account Number = "+accountNumber);
        return account.balance;
    }
        
    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public void deposit(int accountNumber, double amount) {
    	Account account = manager.find(Account.class, accountNumber);
        if(account==null)
        	throw new EJBException("Account not found..!!. Account Number = "+accountNumber);
        double new_balance = account.getBalance() + amount;
        account.setBalance(new_balance);
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)    
    public double withdraw(int accountNumber, double  amount) { 
    	
    	Account account = manager.find(Account.class, accountNumber);
    	
        if(account==null)
        	throw new EJBException("Account not found..!!. Account Number = "+accountNumber);

        if (amount > account.getBalance()) {
            return 0;
        } else {
        	double new_balance = account.getBalance() - amount;
            account.setBalance(new_balance);
            return amount;
        }
    }
   

    
}
